//---------------------------------------------------------------------------
#include <cstdio>
#include <iostream>
#pragma hdrstop
#include "MF_demo_board.h"
#include "MF_defines.h"

//---------------------------------------------------------------------------
#define PM_INTEL    0
#define PM_MOTOROLA 1
#define PM_MCORE    2

/*
  For this to work:
  CLK_Rx XCO: 40 MHz
  Jumpers: Ext Encode (1-2), CLKRx (1-2), External Encode rx: down (closed)
  FM antenna is connected to "Channel 1"
*/

using namespace std;

const double xco_frq=40e6; //MHz

struct MF_reg_
{
  int addr;
  int value;
};

struct MF_reg_ config [] ={    // MF01 configuration: {reg address, value}
{0x0009, 0x0000},  // ADC_CFG
{0x0010, 0x0000},  // RX_CFG
{0x0014, 0x0014},  // RX_FIFO_THRESH
{0x0015, 0x0001},  // RX_LINK, Link port enabled, 8 bit SHARC
{0x0022, 0xc01f},  // RX1_LCFG
{0x0024, 0x130a},  // RX1_DCIC2
{0x0025, 0x601b},  // RX1_DCICN
{0x0026, 0x1002},  // RX1_DFIR1_CFG1
{0x0027, 0x0005},  // RX1_DFIR1_CFG2
{0x0028, 0x2801},  // RX1_DFIR2_CFG1
{0x0029, 0x0004},  // RX1_DFIR2_CFG2
{0x002a, 0x4000},  // RX1_GAIN_I
{0x002b, 0x0000},  // RX1_GAIN_Q
{0x002d, 0x0000},  // RX1_NCO_PHASE
{0x002e, 0x0064},  // RX1_NCO_FRQL
{0x002f, 0x0000},  // RX1_NCO_FRQH
{0x0040, 0x00ae},  // RX1_DFIR1 coefficients
{0x0041, 0x0000},
{0x0042, 0x03ca},
{0x0043, 0x0000},
{0x0044, 0x0763},
{0x0045, 0x0000},
{0x0046, 0x00d7},
{0x0047, 0x0000},
{0x0048, 0xec91},
{0x0049, 0x0000},
{0x004a, 0xe8dd},
{0x004b, 0x0000},
{0x004c, 0x2004},
{0x004d, 0x0000},
{0x004e, 0x7fff},
{0x004f, 0x0000},
{0x0050, 0x58d7},
{0x0051, 0x0000},
{0x0080, 0x0020},  //  RX1_DFIR2 coefficients
{0x0081, 0x0000},
{0x0082, 0x0093},
{0x0083, 0x0000},
{0x0084, 0x0135},
{0x0085, 0x0000},
{0x0086, 0x0134},
{0x0087, 0x0000},
{0x0088, 0xffcb},
{0x0089, 0x0000},
{0x008a, 0xfdd6},
{0x008b, 0x0000},
{0x008c, 0xfdf4},
{0x008d, 0x0000},
{0x008e, 0x0165},
{0x008f, 0x0000},
{0x0090, 0x04d3},
{0x0091, 0x0000},
{0x0092, 0x02ac},
{0x0093, 0x0000},
{0x0094, 0xfb01},
{0x0095, 0x0000},
{0x0096, 0xf6ee},
{0x0097, 0x0000},
{0x0098, 0xff0e},
{0x0099, 0x0000},
{0x009a, 0x0d43},
{0x009b, 0x0000},
{0x009c, 0x0e2a},
{0x009d, 0x0000},
{0x009e, 0xf83e},
{0x009f, 0x0000},
{0x00a0, 0xe075},
{0x00a1, 0x0000},
{0x00a2, 0xed92},
{0x00a3, 0x0000},
{0x00a4, 0x2e83},
{0x00a5, 0x0000},
{0x00a6, 0x7fff},
{0x00a7, 0x0000},
{0x00a8, 0x5297},
{0x00a9, 0x0000},
{0x0122, 0x401f},  // channels 2-4 are off
{0x0222, 0x401f},
{0x0322, 0x401f},
{-1, 0} };         // end of data

#pragma argsused
int main(int argc, char* argv[])
{
  short *hs_buf=new short[8192];

  Board_Descriptor_ bd;   // First, create board descriptor.
                          //Default values are OK
  // Create board interface class instance sing this bd
  MF_demo_board_ *brd=MF_demo_board_::create(bd);
  if(!brd)
  {
    cerr<<"Couldn't find the board\n";
    return 1;
  }

  FILE *outf=fopen("data_received.txt", "w");
  if(!outf)
  {
    cerr<<"Couldn't open output file for writing\n";
    return 1;
  }

  // first, reset the board
  // Argument is ignored
  brd->reset(10);

  brd->get_board_descriptor(bd); // optional, get board info
  cout<<bd; // print it

  // Setup parallel port: P32=0, PSYNC=0,
  // PMODE can be any of PM_INTEL, PM_MOTOROLA, PM_MCORE,
  // PP_timeout recommended value is 15
  brd->setup_PPort(0, 0, PM_INTEL, 15);
  brd->set_Control_Serial(0);  // 0 - PPort, 1 - SPort ( 1 is being debugged)
  brd->set_Optimize_Access(0); // 0/1
  brd->set_LEDs(0, 3); // both off
  brd->set_LEDs(1, 1); // red on
  // Optionally, assign NUM in range 0..7. Default is 0
  brd->MF_set_NUM(3);

  // Reset MF after NUM and PMODE change
  // Duration is in ns, max ~250ns
  brd->MF_reset(200);

  // You can write and read RIA, RID, RC, RD explicitly, if needed
//  brd->MF_write_RC(4);

  cout<<brd->MF_read_reg(ADDR_DEVID)<<endl; // Display MF Device ID (0x101=257)

  for(int a=0; config[a].addr>=0; a++) // loading configuration to MF
    brd->MF_write_reg(config[a].addr, config[a].value);

  // Adjust board's background data transfer mode to match the MF configuration
  brd->set_data_mode(MF_demo_board_::Link8);

  // Tuning channel 1 NCO to 105.7 MHz (6th Nyquist zone for Fs=40 MHz)
  unsigned int frq=105.7e6/xco_frq*0x100000000L;
  brd->MF_write_reg(ADDR_RX1_NCO_FRQL, frq&0xffff);
  brd->MF_write_reg(ADDR_RX1_NCO_FRQH, (frq>>16)&0xffff);


  // Applying new values for NCO, FGAIN, then clearing RX path
  brd->MF_write_reg(ADDR_SYNC, (1<<RFLD_SYNC_master_sync)|(1<<RFLD_SYNC_sync_nco));
  brd->MF_write_reg(ADDR_SYNC, (1<<RFLD_SYNC_master_sync)|(1<<RFLD_SYNC_sync_gain));
  brd->MF_write_reg(ADDR_SYNC, (1<<RFLD_SYNC_master_sync)|(1<<RFLD_SYNC_clr));

  // enabling high speed background data transfer
  brd->HS_enable(1, 0);

  // Starting data processing
  brd->MF_write_reg(ADDR_SYNC, (1<<RFLD_SYNC_master_sync)|(1<<RFLD_SYNC_start_stop));

  brd->set_LEDs(0, 3); // both off
  brd->set_LEDs(1, 2); // green on

  // saving 256K complex samples to file
  for(int a=0; a<0x40000; a+=4096)
  {
    // Wait while 4K samples are available
    while(brd->samples_available()<4096)
      Sleep(10);

    // Read samples. They are formatted as {short I, short Q}
    // You can also use MF_read_sample(), but this is faster:
    brd->HS_read(0, 16384, (char *)hs_buf);

    for(int b=0; b<8192; b+=2)
      fprintf(outf, "%d %d\n", hs_buf[b], hs_buf[b+1]);
  }
  fclose(outf);
  brd->set_LEDs(0, 3); // both off

  // Stopping MF
  brd->MF_write_reg(ADDR_SYNC, 0);

  // Disabling data transfer
  brd->HS_enable(0, 0);

  // Check bit 14 of the INT register
  if(brd->MF_read_reg(ADDR_INT)&(1<<RFLD_INT_rx_fifo_err))
    cout<<"FIFO overflow occured, some data is lost :(\n";

  return 0;
}
//---------------------------------------------------------------------------
